from PyQt6.QtCore import QCoreApplication, QObject


class WeatherConditions(QObject):

    def __init__(self, parent=None):
        super(WeatherConditions, self).__init__(parent)
        self.trans = {'200': self.tr('thunderstorm with light rain'),
                      '201': self.tr('thunderstorm with rain'),
                      '202': self.tr('thunderstorm with heavy rain'),
                      '210': self.tr('light thunderstorm'),
                      '211': self.tr('thunderstorm'),
                      '212': self.tr('heavy thunderstorm'),
                      '221': self.tr('ragged thunderstorm'),
                      '230': self.tr('thunderstorm with light drizzle'),
                      '231': self.tr('thunderstorm with drizzle'),
                      '232': self.tr('thunderstorm with heavy drizzle'),
                      '300': self.tr('light intensity drizzle'),
                      '301': self.tr('drizzle'),
                      '302': self.tr('heavy intensity drizzle'),
                      '310': self.tr('light intensity drizzle rain'),
                      '311': self.tr('drizzle rain'),
                      '312': self.tr('heavy intensity drizzle rain'),
                      '313': self.tr('shower rain and drizzle'),
                      '314': self.tr('heavy shower rain and drizzle'),
                      '321': self.tr('shower drizzle'),
                      '500': self.tr('light rain'),
                      '501': self.tr('moderate rain'),
                      '502': self.tr('heavy intensity rain'),
                      '503': self.tr('very heavy rain'),
                      '504': self.tr('extreme rain'),
                      '511': self.tr('freezing rain'),
                      '520': self.tr('light intensity shower rain'),
                      '521': self.tr('shower rain'),
                      '522': self.tr('heavy intensity shower rain'),
                      '531': self.tr('ragged shower rain'),
                      '600': self.tr('light snow'),
                      '601': self.tr('snow'),
                      '602': self.tr('heavy snow'),
                      '611': self.tr('sleet'),
                      '612': self.tr('shower sleet'),
                      '615': self.tr('light rain and snow'),
                      '616': self.tr('rain and snow'),
                      '620': self.tr('light shower snow'),
                      '621': self.tr('shower snow'),
                      '622': self.tr('heavy shower snow'),
                      '701': self.tr('mist'),
                      '711': self.tr('smoke'),
                      '721': self.tr('haze'),
                      '731': self.tr('sand, dust whirls'),
                      '741': self.tr('fog'),
                      '751': self.tr('sand'),
                      '761': self.tr('dust'),
                      '762': self.tr('volcanic ash'),
                      '771': self.tr('squalls'),
                      '781': self.tr('tornado'),
                      '800': self.tr('clear sky'),
                      '801': self.tr('few clouds'),
                      '802': self.tr('scattered clouds'),
                      '803': self.tr('broken clouds'),
                      '804': self.tr('overcast clouds'),
                      '900': self.tr('tornado'),
                      '901': self.tr('tropical storm'),
                      '902': self.tr('hurricane'),
                      '903': self.tr('cold'),
                      '904': self.tr('hot'),
                      '905': self.tr('windy'),
                      '906': self.tr('hail'),
                      '951': self.tr('calm'),
                      '952': self.tr('light breeze'),
                      '953': self.tr('gentle breeze'),
                      '954': self.tr('moderate breeze'),
                      '955': self.tr('fresh breeze'),
                      '956': self.tr('strong breeze'),
                      '957': self.tr('high wind, near gale'),
                      '958': self.tr('gale'),
                      '959': self.tr('severe gale'),
                      '960': self.tr('storm'),
                      '961': self.tr('violent storm'),
                      '962': self.tr('hurricane')}

        self.clouds = {'clear sky': '800',
                       'few clouds': '801',
                       'scattered clouds': '802',
                       'broken clouds': '803',
                       'overcast clouds': '804'}

        self.wind = {'calm': '951',
                     'light breeze': '952',
                     'gentle breeze': '953',
                     'moderate breeze': '954',
                     'fresh breeze': '955',
                     'strong breeze': '956',
                     'high wind, near gale': '957',
                     'gale': '958',
                     'severe gale': '959',
                     'storm': '960',
                     'violent storm': '961',
                     'hurricane': '962'}

        self.rain = {'no': QCoreApplication.translate(
                     'Precipitation type', 'no', 'Weather overview dialogue'),
                     'rain': QCoreApplication.translate(
                     'Precipitation type', 'rain',
                     'Weather overview dialogue'),
                     'snow': QCoreApplication.translate(
                     'Precipitation type', 'snow',
                     'Weather overview dialogue')
                     }

        self.wind_direction = {'N': self.tr('North'),
                               'NE': self.tr('NorthEast'),
                               'NNE': self.tr('North-northeast'),
                               'NW': self.tr('NorthWest'),
                               'NNW': self.tr('North-northwest'),
                               'S': self.tr('South'),
                               'SE': self.tr('SouthEast'),
                               'SSE': self.tr('South-southeast'),
                               'SW': self.tr('SouthWest'),
                               'SSW': self.tr('South-southwest'),
                               'E': self.tr('East'),
                               'ESE': self.tr('East-southeast'),
                               'ENE': self.tr('East-northeast'),
                               'W': self.tr('West'),
                               'WSW': self.tr('West-southwest'),
                               'WNW': self.tr('West-northwest')}

        self.wind_codes = {'N': self.tr('N'),
                           'NE': self.tr('NE'),
                           'NNE': self.tr('NNE'),
                           'NW': self.tr('NW'),
                           'NNW': self.tr('NNW'),
                           'S': self.tr('S'),
                           'SE': self.tr('SE'),
                           'SSE': self.tr('SSE'),
                           'SW': self.tr('SW'),
                           'SSW': self.tr('SSW'),
                           'E': self.tr('E'),
                           'ESE': self.tr('ESE'),
                           'ENE': self.tr('ENE'),
                           'W': self.tr('W'),
                           'WSW': self.tr('WSW'),
                           'WNW': self.tr('WNW')}

        self.uv_risk = {'Low':
                        QCoreApplication.translate('UV risk', 'Low', ''),
                        'Moderate':
                        QCoreApplication.translate('UV risk',
                                                   'Moderate', ''),
                        'High':
                        QCoreApplication.translate('UV risk', 'High', ''),
                        'Very high':
                        QCoreApplication.translate('UV risk', 'Very high', ''),
                        'Extreme':
                        QCoreApplication.translate('UV risk', 'Extreme', ''),
                        'None': '-'}

        self.uv_recommend = {
            'Low': QCoreApplication.translate(
                'Low UV recommended protection',
                '''Wear sunglasses on bright days; use sunscreen if there is
                snow on<br/>the ground, which reflects UV radiation,
                or if you have particularly fair skin.''',
                'Low https://en.wikipedia.org/wiki/Ultraviolet_index'),
            'Moderate': QCoreApplication.translate(
                'Moderate UV recommended protection',
                '''Take precautions, such as covering up, if you will be
                outside.<br/>Stay in shade near midday
                when the sun is strongest.''',
                'Moderate https://en.wikipedia.org/wiki/Ultraviolet_index'),
            'High': QCoreApplication.translate(
                'High UV recommended protection',
                '''Cover the body with sun protective clothing, use SPF 30+
                sunscreen,<br/>wear a hat, reduce time in the sun within three
                hours of solar noon, and wear sunglasses.''',
                'High https://en.wikipedia.org/wiki/Ultraviolet_index'),
            'Very high': QCoreApplication.translate(
                'Very high UV recommended protection',
                '''Wear SPF 30+ sunscreen, a shirt, sunglasses, and a
                wide-brimmed hat.<br/>Do not stay in the sun for too long.''',
                'Very high https://en.wikipedia.org/wiki/Ultraviolet_index'),
            'Extreme': QCoreApplication.translate(
                'Extreme UV recommended protection',
                '''Take all precautions: Wear SPF 30+ sunscreen, a long-sleeved
                shirt and trousers,<br/>sunglasses, and a very broad hat.
                Avoid the sun within three hours of solar noon.''',
                'Extreme https://en.wikipedia.org/wiki/Ultraviolet_index'),
            'None': '-'
        }

        self.beaufort = {
            '0': QCoreApplication.translate(
                'Beaufort scale 0 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Sea like a mirror\nLand: Calm. Smoke rises vertically',
                'Tooltip in Weather overview dialogue'
            ),
            '1': QCoreApplication.translate(
                'Beaufort scale 1 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Ripples with the appearance of scales are formed,'
                'but without foam crests\nLand: Smoke drift indicates wind '
                'direction. Leaves and wind vanes are stationary',
                'Tooltip in Weather overview dialogue'
            ),
            '2': QCoreApplication.translate(
                'Beaufort scale 2 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Small wavelets, still short but more pronounced; '
                'crests have a glassy appearance and do not break\nLand: '
                'Wind felt on exposed skin. Leaves rustle. Wind vanes begin '
                'to move',
                'Tooltip in Weather overview dialogue'
            ),
            '3': QCoreApplication.translate(
                'Beaufort scale 3 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Large wavelets. Crests begin to break; '
                'scattered whitecaps\nLand: Leaves and small twigs constantly '
                'moving, light flags extended',
                'Tooltip in Weather overview dialogue'
            ),
            '4': QCoreApplication.translate(
                'Beaufort scale 4 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Small waves with breaking crests. Fairly frequent '
                'whitecaps\nLand: Dust and loose paper raised. Small branches '
                'begin to move',
                'Tooltip in Weather overview dialogue'
            ),
            '5': QCoreApplication.translate(
                'Beaufort scale 5 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Moderate waves of some length. Many whitecaps. '
                'Small amounts of spray\nLand: Branches of a moderate size '
                'move. Small trees in leaf begin to sway',
                'Tooltip in Weather overview dialogue'
            ),
            '6': QCoreApplication.translate(
                'Beaufort scale 6 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Long waves begin to form. White foam crests are very '
                'frequent. Some airborne spray is present\nLand: '
                'Large branches in motion. Whistling heard in overhead wires. '
                'Umbrella use becomes difficult. Empty plastic bins tip over',
                'Tooltip in Weather overview dialogue'
            ),
            '7': QCoreApplication.translate(
                'Beaufort scale 7 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Sea heaps up. Some foam from breaking waves is blown '
                'into streaks along wind direction. Moderate amounts of '
                'airborne spray\nLand: Whole trees in motion. Effort needed '
                'to walk against the wind',
                'Tooltip in Weather overview dialogue'
            ),
            '8': QCoreApplication.translate(
                'Beaufort scale 8 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Moderately high waves with breaking crests forming '
                'spindrift. Well-marked streaks of foam are blown along wind '
                'direction. Considerable airborne spray\nLand: Some twigs '
                'broken from trees. Cars veer on road. '
                'Progress on foot is seriously impeded',
                'Tooltip in Weather overview dialogue'
            ),
            '9': QCoreApplication.translate(
                'Beaufort scale 9 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: High waves whose crests sometimes roll over. '
                'Dense foam is blown along wind direction. Large amounts '
                'of airborne spray may begin to reduce visibility\nLand: '
                'Some branches break off trees, and some small trees blow '
                'over. Construction/temporary signs and barricades blow over',
                'Tooltip in Weather overview dialogue'
            ),
            '10': QCoreApplication.translate(
                'Beaufort scale 10 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Very high waves with overhanging crests. '
                'Large patches of foam from wave crests give the sea a white '
                'appearance. Considerable tumbling of waves with heavy '
                'impact. Large amounts of airborne spray reduce visibility\n'
                'Land: Trees are broken off or uprooted, structural damage '
                'likely',
                'Tooltip in Weather overview dialogue'
            ),
            '11': QCoreApplication.translate(
                'Beaufort scale 11 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Exceptionally high waves. Very large patches of foam, '
                'driven before the wind, cover much of the sea surface. Very '
                'large amounts of airborne spray severely reduce visibility\n'
                'Land: Widespread vegetation and structural damage likely',
                'Tooltip in Weather overview dialogue'
            ),
            '12': QCoreApplication.translate(
                'Beaufort scale 12 - https://en.wikipedia.org/wiki/Beaufort_scale#Modern_scale',
                'Sea: Huge waves. Sea is completely white with foam and '
                'spray. Air is filled with driving spray, greatly reducing '
                'visibility\nLand: Severe widespread damage to '
                'vegetation and structures. Debris and unsecured objects '
                'are hurled about',
                'Tooltip in Weather overview dialogue')
        }
        self.pressure = {
            'hpa': QCoreApplication.translate(
                'From the site: Why hPa Is Used'
                ' - http://n-a-c.biz/WS62694/hPa.htm',
                'If the barometric pressure is rising, the weather is '
                'improving.\nIf the barometric pressure is falling, '
                'the weather is worsening.\nAbove 1020 hPa is calm '
                'weather/cooler temperatures and below that is unsettled '
                'weather/warmer temperatures.\n1010-1020 hPa would be weak '
                'to normal storms.\n990-1010 hPa would be bad storms with '
                'possible hurricanes, tornadoes, and typhoons.\nBelow 990 hPa '
                'is the pressure for VERY BAD and POWERFUL hurricanes, '
                'tornadoes, and typhoons.',
                'Tooltip for the pressure label in weather overview'
            )
        }



#!/usr/bin/env python3

from distutils.command.build import build
from distutils.core import setup
from pathlib import Path
import re
import subprocess


class BuildQm(build):
    # qt5 : os.system('pyrcc5 -o meteo_qt/qrc_resources.py meteo_qt/resources.qrc')
    # qt6 (PySide2/6) : os.system('rcc -g python meteo_qt/resources.qrc | sed '0,/PySide2/s//PyQt6/' > meteo_qt/qrc_resources.py')

    def extract_py_sources(project_file='meteo_qt/meteo_qt.pro'):
        with open(project_file, "r", encoding="utf-8") as file:
            content = file.read()
            pyproject_files = ["meteo_qt/" + py for py in re.findall(r"([a-zA-Z0-9_/]+\.py)", content)]
            return pyproject_files

    subprocess.run(["pylupdate6"] + extract_py_sources() + ["-ts", "meteo_qt/translations/meteo-qt_en.ts"])
    subprocess.run(["lrelease-pro-qt6", "meteo_qt/meteo_qt.pro"])


PROJECT_PATH = Path(__file__).parent
with (PROJECT_PATH / "requirements.txt").open() as f:
    install_requires = [line.strip() for line in f.readlines()]

setup(
    name='meteo_qt',
    version='4.2',
    description='A system tray application for the weather status',
    author='Dimitrios Glentadakis',
    author_email='dglent@free.fr',
    url='https://github.com/dglent/meteo-qt',
    license='GPLv3',
    packages=['meteo_qt'],
    install_requires=install_requires,
    keywords=['weather', 'qt', 'trayicon', 'openweathermap', 'forecast'],
    data_files=[('/usr/share/applications', ['share/meteo-qt.desktop']),
                ('/usr/share/icons', ['meteo_qt/images/weather-few-clouds.png']),
                ('/usr/share/meteo_qt/translations',
                    ['meteo_qt/translations/meteo-qt_bg.qm',
                     'meteo_qt/translations/meteo-qt_ca.qm',
                     'meteo_qt/translations/meteo-qt_cs.qm',
                     'meteo_qt/translations/meteo-qt_da.qm',
                     'meteo_qt/translations/meteo-qt_de.qm',
                     'meteo_qt/translations/meteo-qt_el.qm',
                     'meteo_qt/translations/meteo-qt_en.qm',
                     'meteo_qt/translations/meteo-qt_es.qm',
                     'meteo_qt/translations/meteo-qt_fi.qm',
                     'meteo_qt/translations/meteo-qt_fr.qm',
                     'meteo_qt/translations/meteo-qt_he.qm',
                     'meteo_qt/translations/meteo-qt_hu.qm',
                     'meteo_qt/translations/meteo-qt_it.qm',
                     'meteo_qt/translations/meteo-qt_ja.qm',
                     'meteo_qt/translations/meteo-qt_lt.qm',
                     'meteo_qt/translations/meteo-qt_nb.qm',
                     'meteo_qt/translations/meteo-qt_nl.qm',
                     'meteo_qt/translations/meteo-qt_pl.qm',
                     'meteo_qt/translations/meteo-qt_pt_BR.qm',
                     'meteo_qt/translations/meteo-qt_pt.qm',
                     'meteo_qt/translations/meteo-qt_ro.qm',
                     'meteo_qt/translations/meteo-qt_ru.qm',
                     'meteo_qt/translations/meteo-qt_sk.qm',
                     'meteo_qt/translations/meteo-qt_sv.qm',
                     'meteo_qt/translations/meteo-qt_tr.qm',
                     'meteo_qt/translations/meteo-qt_uk.qm',
                     'meteo_qt/translations/meteo-qt_vi.qm',
                     'meteo_qt/translations/meteo-qt_zh_CN.qm',
                     'meteo_qt/translations/meteo-qt_zh_TW.qm']),
                ('/usr/share/doc/meteo-qt',
                    ['README.md', 'LICENSE', 'CHANGELOG', 'TODO'])],
        scripts=["bin/meteo-qt"],
        cmdclass={'build_qm': BuildQm},
        include_package_data=True,
        zip_safe=True,
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Other Audience',
            'Natural Language :: English',
            'License :: OSI Approved :: GNU General Public License (GPL)',
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: Implementation :: CPython',
    ],
)



from PyQt6.QtCore import QSize, QCoreApplication
from PyQt6.QtGui import QPixmap, QTextCursor, QIcon
from PyQt6.QtWidgets import (
    QDialog, QDialogButtonBox, QHBoxLayout, QLabel,
    QTabWidget, QTextBrowser, QVBoxLayout
)


class AboutDialog(QDialog):
    def __init__(self, title, text, image, parent=None):
        super(AboutDialog, self).__init__(parent)
        layout = QVBoxLayout()
        titleLayout = QHBoxLayout()
        name_versionLabel = QLabel(title)
        contentsLayout = QHBoxLayout()
        aboutBrowser = QTextBrowser()
        aboutBrowser.append(text)
        aboutBrowser.setOpenExternalLinks(True)
        creditsBrowser = QTextBrowser()
        creditsBrowser.append(self.contributors())
        creditsBrowser.setOpenExternalLinks(True)
        TabWidget = QTabWidget()
        TabWidget.addTab(aboutBrowser, self.tr('About'))
        TabWidget.addTab(creditsBrowser, self.tr('Contributors'))
        aboutBrowser.moveCursor(QTextCursor.MoveOperation.Start)
        creditsBrowser.moveCursor(QTextCursor.MoveOperation.Start)
        imageLabel = QLabel()
        icon = QIcon.fromTheme('weather-few-clouds')
        if icon.isNull():
            imageLabel.setPixmap(QPixmap(image))
        else:
            imageLabel.setPixmap(icon.pixmap(48, 48))
        titleLayout.addWidget(imageLabel)
        titleLayout.addWidget(name_versionLabel)
        titleLayout.addStretch()
        contentsLayout.addWidget(TabWidget)
        buttonLayout = QHBoxLayout()
        buttonBox = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok)
        buttonLayout.addWidget(buttonBox)
        layout.addLayout(titleLayout)
        layout.addLayout(contentsLayout)
        layout.addLayout(buttonLayout)
        self.setLayout(layout)
        buttonBox.clicked.connect(self.accept)
        self.setMinimumSize(QSize(380, 400))
        self.setWindowTitle(self.tr('About Meteo-qt'))

    def contributors(self):

        contributors = (
            QCoreApplication.translate(
                '',
                'Jiri Podhorecky<br/>'
                'Pavel Fric<br/>'
                '[cs] Czech translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Jürgen Thurau <a href="mailto:linux@psyca.de">linux@psyca.de</a>'
                '<br/>[de] German translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Peter Mattern '
                '<a href="mailto:pmattern@arcor.de">pmattern@arcor.de</a>'
                '<br/> [de] German translation, Project',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Dimitrios Glentadakis '
                '<a href="mailto:dglent@free.fr">dglent@free.fr</a>'
                '<br/> [el] Greek translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p> juancarlospaco '
                '<a href="mailto:JuanCarlosPaco@gmail.com">'
                'JuanCarlosPaco@gmail.com</a>'
                '<br/> [es] Spanish translation, Project',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Ozkar L. Garcell '
                '<a href="mailto:ozkar.garcell@gmail.com">'
                'ozkar.garcell@gmail.com</a>'
                '<br/> Teo Laírla (teolairlasg) '
                '<a href="mailto:teo.lairla@iessierradeguara.com">'
                'teo.lairla@iessierradeguara.com</a>'
                '<br/> [es] Spanish translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Laurene Albrand '
                '<a href="mailto:laurenealbrand@outlook.com">'
                'laurenealbrand@outlook.com</a>'
                '<br/> [fr] French translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Rémi Verschelde '
                '<a href="mailto:remi@verschelde.fr">remi@verschelde.fr</a>'
                '<br/> [fr] French translation, Project',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>werthad <a href="mailto:werthad@gmail.com">'
                'werthad@gmail.com</a>'
                '<br/> [hu] Hungarian translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Archita Zabaleta <a href="mailto:architazabaleta@gmail.com">'
                'architazabaleta@gmail.com</a>'
                '<br/> [it] Italian translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>standreas <a href="mailto:standreas@riseup.net">'
                'standreas@riseup.net</a>'
                '<br/> [it] Italian translation, Weblate',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Masamichi Ito <a href="https://github.com/ito32bit">'
                'https://github.com/ito32bit</a>'
                '<br/> [ja] Japanese translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Heimen Stoffels '
                '<a href="mailto:vistausss@outlook.com">'
                'vistausss@outlook.com</a>'
                '<br/> [nl] Dutch translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Daniel Napora '
                '<a href="mailto:napcok@gmail.com">napcok@gmail.com</a>'
                '<br/> Tomasz Przybył '
                '<a href="mailto:fademind@gmail.com">fademind@gmail.com</a>'
                '<br/> [pl] Polish translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Adrian Moise'
                '<a href="mailto:sah.mat.ro@gmail.com">sah.mat.ro@gmail.com</a>'
                '<br/> [ro] Romanian translation',
                ''
            )

            + QCoreApplication.translate(
                '',
                '<p>Artem Vorotnikov '
                '<a href="mailto:artem@vorotnikov.me">artem@vorotnikov.me</a>'
                '<br/> Sergey Shitikov '
                '<a href="mailto:rw4lll@yandex.ru">rw4lll@yandex.ru</a>'
                '<br/>Alexey Zakaldaev '
                '<a href="mailto:nelex111@gmail.com">nelex111@gmail.com</a>'
                '<br/>Liliya Panova<br/> [ru] Russian translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>jose1711 '
                '<a href="mailto:jose1711@gmail.com">'
                'jose1711@gmail.com</a>'
                '<br> [sk] Slovak translation, Project',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Luna bittin Jernberg '
                '<br> [sv] Swedish translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Atilla Öntaş '
                '<a href="mailto:tarakbumba@gmail.com">'
                'tarakbumba@gmail.com</a>'
                '<br/> [tr] Turkish translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>Yuri Chornoivan '
                '<a href="mailto:yurchor@ukr.net">yurchor@ukr.net</a>'
                '<br/> [uk] Ukrainian translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>You-Cheng Hsieh '
                '<a href="mailto:yochenhsieh@gmail.com">'
                'yochenhsieh@gmail.com</a>'
                '<br/> [zh_TW] Chinese (Taiwan) translation',
                ''
            )
            + QCoreApplication.translate(
                '',
                '<p>pmav99<br/> Project',
                ''
            )
        )
        return contributors



from PyQt6.QtCore import QCoreApplication, Qt, QSettings, pyqtSignal
from PyQt6.QtWidgets import (QDialog, QDialogButtonBox, QHBoxLayout, QLabel,
                             QListWidget, QPushButton, QVBoxLayout)

try:
    import searchcity
    import citytranslate
except ImportError:
    from meteo_qt import searchcity
    from meteo_qt import citytranslate


class CityListDlg(QDialog):
    citieslist_signal = pyqtSignal([list])
    citiesdict_signal = pyqtSignal([dict])

    def __init__(
        self, citylist, accurate_url, appid, trans_cities_dict, parent=None
    ):
        super(CityListDlg, self).__init__(parent)
        self.settings = QSettings()
        self.citylist = citylist
        self.trans_cities_dict = trans_cities_dict
        self.accurate_url = accurate_url
        self.appid = appid
        self.listWidget = QListWidget()
        self.listWidget.itemDoubleClicked.connect(self.translate)
        cities_list = []
        for i in self.citylist:
            cities_list.append(self.trans_cities_dict.get(i, i))
        self.listWidget.addItems(cities_list)
        buttonLayout = QVBoxLayout()
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
        )
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.accepted.connect(self.accept)
        layoutT = QVBoxLayout()
        layout = QHBoxLayout()
        layout.addWidget(self.listWidget)
        layout.addLayout(buttonLayout)
        for text, slot in ((self.tr("&Add..."), self.add),
                           (self.tr("&Remove..."), self.remove),
                           (self.tr("&Up"), self.up),
                           (self.tr("&Down"), self.down),
                           (self.tr("De&fault"), self.default),
                           (self.tr("&Sort"), self.listWidget.sortItems)):
            button = QPushButton(text)
            buttonLayout.addWidget(button)
            button.clicked.connect(slot)
        self.translate_button = QPushButton(
            QCoreApplication.translate(
                'Button',
                '&Translate',
                'Edit cities name'
            )
        )
        buttonLayout.addWidget(self.translate_button)
        self.translate_button.clicked.connect(self.translate)
        buttonLayout.addWidget(self.buttonBox)
        self.status = QLabel()
        layoutT.addLayout(layout)
        layoutT.addWidget(self.status)
        self.setLayout(layoutT)
        self.setWindowTitle(
            QCoreApplication.translate(
                'Window title',
                'Cities',
                'Cities list dialogue'
            )
        )
        self.checklength()

    def add(self):
        self.status.setText('')
        lista = []
        newitem = ''
        self.citytoadd = ''
        self.countrytoadd = ''
        self._idtoadd = ''
        dialog = searchcity.SearchCity(self.accurate_url, self.appid, self)
        dialog.id_signal.connect(self.addcity)
        dialog.city_signal.connect(self.addcity)
        dialog.country_signal.connect(self.addcity)
        if dialog.exec() == 1:
            newitem = (
                self.citytoadd + '_' + self.countrytoadd
                + '_' + self._idtoadd
            )
            for row in range(self.listWidget.count()):
                lista.append(self.listWidget.item(row).text())
            if newitem in lista:
                self.status.setText(
                    QCoreApplication.translate(
                        'Status bar message',
                        'The city already exists in the list',
                        'Cities list dialogue'
                    )
                )
                return
            else:
                self.listWidget.addItem(newitem)
                self.checklength()
                self.status.setText(
                    'ℹ ' + QCoreApplication.translate(
                        'Status bar message',
                        'Toggle cities with mouse scroll on the weather window',
                        'Cities list dialogue'
                    )
                )

    def addcity(self, what):
        self.status.setText('')
        if what[0] == 'ID':
            self._idtoadd = what[1]
        elif what[0] == 'City':
            self.citytoadd = what[1]
        elif what[0] == 'Country':
            self.countrytoadd = what[1]

    def remove(self):
        self.status.setText('')
        if self.listWidget.count() == 1:
            self.status.setText(
                QCoreApplication.translate(
                    'Message when trying to remove the'
                    'last and unique city in the list',
                    'This is the default city!',
                    'Cities list dialogue'
                )
            )
            return
        row = self.listWidget.currentRow()
        item = self.listWidget.item(row)
        if item is None:
            return
        message = self.tr('The city "{0}" has been removed').format(
            self.listWidget.item(row).text())
        item = self.listWidget.takeItem(row)
        del item
        self.status.setText(message)

    def up(self):
        self.status.setText('')
        row = self.listWidget.currentRow()
        if row >= 1:
            item = self.listWidget.takeItem(row)
            self.listWidget.insertItem(row - 1, item)
            self.listWidget.setCurrentItem(item)

    def down(self):
        self.status.setText('')
        row = self.listWidget.currentRow()
        if row < self.listWidget.count() - 1:
            item = self.listWidget.takeItem(row)
            self.listWidget.insertItem(row + 1, item)
            self.listWidget.setCurrentItem(item)

    def default(self):
        self.status.setText('')
        row = self.listWidget.currentRow()
        if row >= 1:
            item = self.listWidget.takeItem(row)
            self.listWidget.insertItem(0, item)
            self.listWidget.setCurrentItem(item)

    def checklength(self):
        if self.listWidget.count() == 1:
            # After adding the first city the entry is not activated
            self.listWidget.setCurrentRow(0)
        if self.listWidget.count() > 0:
            self.translate_button.setEnabled(True)
            self.listWidget.setMinimumWidth(
                self.listWidget.sizeHintForColumn(0)
            )
        else:
            self.translate_button.setEnabled(False)

    def translate(self):
        city = self.listWidget.currentItem().text()
        dialog = citytranslate.CityTranslate(
            city, self.trans_cities_dict, self
        )
        dialog.city_signal.connect(self.current_translation)
        if dialog.exec() == 1:
            row = self.listWidget.currentRow()
            item = self.listWidget.takeItem(row)
            del item
            self.listWidget.insertItem(row, self.current_translated_city)
            self.listWidget.setCurrentRow(row)

    def current_translation(self, translated_city):
        for city, translated in translated_city.items():
            if translated == '':
                translated = city
            self.trans_cities_dict[city] = translated
            self.current_translated_city = translated

    def accept(self):
        listtosend = []
        for row in range(self.listWidget.count()):
            city = self.find_city_key(self.listWidget.item(row).text())
            listtosend.append(city)
        if self.listWidget.count() == 0:
            return
        self.citieslist_signal[list].emit(listtosend)
        self.citiesdict_signal[dict].emit(self.trans_cities_dict)
        QDialog.accept(self)

    def find_city_key(self, city):
        for key, value in self.trans_cities_dict.items():
            if value == city:
                return key
        return city



import logging
import os
import glob

from PyQt6.QtCore import (
    QCoreApplication, QLocale, QSettings, QSize, Qt, pyqtSignal
)
from PyQt6.QtGui import QIcon, QColor, QFont
from PyQt6.QtWidgets import (
    QCheckBox, QColorDialog, QComboBox, QDialog, QDialogButtonBox,
    QGridLayout, QHBoxLayout, QLabel, QLineEdit, QPushButton, QSpinBox,
    QVBoxLayout, QFontDialog
)

try:
    import citylistdlg
    import proxydlg
except ImportError:
    from meteo_qt import citylistdlg
    from meteo_qt import proxydlg


class MeteoSettings(QDialog):
    applied_signal = pyqtSignal()

    def __init__(self, accurate_url, appid, parent=None):
        super(MeteoSettings, self).__init__(parent)
        self.settings = QSettings()
        trans_cities_dict = self.settings.value('CitiesTranslation') or '{}'
        self.trans_cities_dict = eval(trans_cities_dict)
        self.layout = QVBoxLayout()
        self.accurate_url = accurate_url
        self.appid = appid
        self.set_city = self.settings.value('City') or '?'
        locale = QLocale.system().name()
        locale_long = ['en_GB', 'pt_BR', 'zh_CN', 'zh_TW']
        if locale not in locale_long:
            locale = locale[:2]
        self.interval_set = self.settings.value('Interval') or '30'
        self.temp_tray_color = self.settings.value('TrayColor') or ''
        # -----Cities comboBox------------------------
        self.first = True
        self.clear_combo = False
        self.city_list_before = []
        self.citylist = []
        self.city_combo = QComboBox()
        if self.set_city != '?':
            self.add_cities_incombo()
        self.city_combo.currentIndexChanged.connect(self.city_default)
        self.city_title = QLabel(self.tr('City'))
        self.city_button = QPushButton()
        self.city_button.setIcon(QIcon(':/configure'))
        self.city_button.setToolTip(self.tr('Click to edit the cities list'))
        self.city_button.clicked.connect(self.edit_cities_list)
        # ------Language------------------------------
        self.language_label = QLabel(self.tr('Language'))
        self.language_combo = QComboBox()
        self.language_combo.setToolTip(
            QCoreApplication.translate(
                'Tooltip',
                'The application has to be restared to apply the language setting',
                'Settings dialogue'
            )
        )
        self.language_dico = {
            'bg': self.tr('Bulgarian'),
            'ca': self.tr('Catalan'),
            'cs': self.tr('Czech'),
            'da': self.tr('Danish'),
            'de': self.tr('German'),
            'el': self.tr('Greek'),
            'en_GB': self.tr('English GB'),
            'en': self.tr('English'),
            'es': self.tr('Spanish'),
            'et': self.tr('Estonien'),
            'fi': self.tr('Finnish'),
            'fr': self.tr('French'),
            'he': self.tr('Hebrew'),
            'hi': self.tr('Hindi'),
            'hr': self.tr('Croatian'),
            'hu': self.tr('Hungarian'),
            'it': self.tr('Italian'),
            'ja': self.tr('Japanese'),
            'ka': self.tr('Georgian'),
            'kab': self.tr('Kabyle'),
            'lt': self.tr('Lithuanian'),
            'nb': self.tr('Norwegian (Bokmaal)'),
            'nl': self.tr('Dutch'),
            'pl': self.tr('Polish'),
            'pt': self.tr('Portuguese'),
            'pt_BR': self.tr('Brazil Portuguese'),
            'ro': self.tr('Romanian'),
            'ru': self.tr('Russian'),
            'sk': self.tr('Slovak'),
            'sv': self.tr('Swedish'),
            'tr': self.tr('Turkish'),
            'uk': self.tr('Ukrainian'),
            'zh_TW': self.tr('Chinese Traditional'),
            'zh_CN': self.tr('Chinese Simplified')
        }
        lang_list = sorted(self.language_dico.values())
        # English as fallback language
        if locale not in self.language_dico:
            locale = 'en'
        self.setLanguage = self.settings.value('Language') or locale
        self.language_combo.addItems(lang_list)
        self.language_combo.setCurrentIndex(
            self.language_combo.findText(self.language_dico[self.setLanguage])
        )
        self.language_combo.currentIndexChanged.connect(self.language)
        self.lang_changed = False
        # Unit system
        self.units_changed = False
        self.temp_unit = self.settings.value('Unit')
        if self.temp_unit is None or self.temp_unit == '':
            self.temp_unit = 'metric'
            self.units_changed = True
        self.units_label = QLabel(self.tr('Temperature unit'))
        self.units_combo = QComboBox()
        self.units_dico = {'metric': '°C', 'imperial': '°F', ' ': '°K'}
        units_list = sorted(self.units_dico.values())
        self.units_combo.addItems(units_list)
        self.units_combo.setCurrentIndex(self.units_combo.findText(
            self.units_dico[self.temp_unit]))
        self.units_combo.currentIndexChanged.connect(self.units)
        # Wind unit
        self.wind_unit_changed = False
        self.wind_unit = self.settings.value('Wind_unit')
        if self.wind_unit is None or self.wind_unit == '':
            self.wind_unit = 'df'
            self.wind_unit_changed = True
        self.wind_unit_label = QLabel(
            QCoreApplication.translate(
                'Label of the checkbox',
                'Wind speed unit',
                'Settings dialogue'
            )
        )
        self.wind_unit_combo = QComboBox()
        wind_units_list = [
            QCoreApplication.translate(
                'Option to choose the default wind speed unit',
                'Default',
                'Settings diaogue'
            ),
            QCoreApplication.translate(
                'Option to choose wind speed unit',
                'Beaufort',
                'Settings dialogue'
            ),
            QCoreApplication.translate(
                'Option to choose wind speed unit',
                'Km/h',
                'Settings dialogue'
            )
        ]
        self.wind_unit_dico = {
            'df': wind_units_list[0],
            'bf': wind_units_list[1],
            'km': wind_units_list[2]
        }

        self.wind_unit_combo.addItems(wind_units_list)
        self.wind_unit_combo.setCurrentIndex(
            self.wind_unit_combo.findText(self.wind_unit_dico[self.wind_unit])
        )
        self.wind_unit_combo.currentIndexChanged.connect(self.wind_unit_change_apply)
        self.wind_unit_combo.model().item(2).setEnabled(False)
        if self.temp_unit == 'metric':
            self.wind_unit_combo.model().item(2).setEnabled(True)
        # Decimal in trayicon
        self.temp_decimal_label = QLabel(
            QCoreApplication.translate(
                'If the temperature will be shown with a decimal or rounded in tray icon',
                'Temperature accuracy in system tray',
                'Settings dialogue'
            )
        )
        self.temp_decimal_combo = QComboBox()
        temp_decimal_combo_dico = {'False': '0°', 'True': '0.1°'}
        temp_decimal_combo_list = [
            temp_decimal_combo_dico['False'], temp_decimal_combo_dico['True']
        ]
        self.temp_decimal_combo.addItems(temp_decimal_combo_list)
        temp_decimal_bool_str = self.settings.value('Decimal') or 'False'
        self.temp_decimal_combo.setCurrentIndex(
            self.temp_decimal_combo.findText(
                temp_decimal_combo_dico[temp_decimal_bool_str]
            )
        )
        self.temp_decimal_combo.currentIndexChanged.connect(self.temp_decimal)
        self.temp_decimal_changed = False
        # Interval of updates
        self.interval_label = QLabel(self.tr('Update interval'))
        self.interval_min = QLabel(self.tr('minutes'))
        self.interval_combo = QComboBox()
        self.interval_list = ['15', '30', '45', '60', '90', '120']
        self.interval_combo.addItems(self.interval_list)
        self.interval_combo.setCurrentIndex(self.interval_combo.findText(
            self.interval_list[self.interval_list.index(self.interval_set)]))
        self.interval_combo.currentIndexChanged.connect(self.interval)
        self.interval_changed = False
        # OK Cancel Apply Buttons
        self.buttonLayout = QHBoxLayout()
        self.buttonLayout.addStretch()
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Apply | QDialogButtonBox.StandardButton.Cancel
        )
        self.buttonBox.setContentsMargins(0, 30, 0, 0)
        self.buttonLayout.addWidget(self.buttonBox)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).clicked.connect(self.apply_settings)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(False)
        # Autostart
        self.autostart_label = QLabel(self.tr('Launch at startup'))
        self.autostart_checkbox = QCheckBox()
        autostart_bool = self.settings.value('Autostart') or 'False'
        autostart_bool = eval(autostart_bool)
        self.autostart_checkbox.setChecked(autostart_bool)
        self.autostart_checkbox.stateChanged.connect(self.autostart)
        self.autostart_changed = False
        # Tray temp° color
        self.temp_colorLabel = QLabel(self.tr('Font colour in the tray'))
        self.temp_colorButton = QPushButton()
        self.temp_colorButton.setStyleSheet(
            'QWidget {{ background-color: {0} }}'.format(self.temp_tray_color))
        self.temp_colorButton.setMaximumSize(QSize(44, 24))
        self.temp_colorButton.clicked.connect(self.color_chooser)
        self.temp_color_resetButton = QPushButton(self.tr('Reset'))
        self.temp_color_resetButton.setToolTip(
            self.tr('Reset font colour to system default'))
        self.temp_color_resetButton.clicked.connect(self.color_reset)
        # Display notifications
        self.notifier_label = QLabel(self.tr('Notification on weather update'))
        self.notifier_checkbox = QCheckBox()
        notifier_bool = self.settings.value('Notifications') or 'True'
        notifier_bool = eval(notifier_bool)
        self.notifier_checkbox.setChecked(notifier_bool)
        self.notifier_checkbox.stateChanged.connect(self.notifier)
        self.notifier_changed = False
        # Icon & Temp
        self.tray_icon_temp_label = QLabel(
            QCoreApplication.translate(
                'Settings dialogue',
                'System tray icon',
                'Setting to choose the type of the icon on the tray'
                '(only icon only text, icon&text'
            )
        )
        self.tray_icon_combo = QComboBox()
        tray_icon_temp = QCoreApplication.translate(
            'Settings dialogue',
            'Icon & temperature',
            'Setting to choose the type of the icon on the tray'
        )
        tray_icon = QCoreApplication.translate(
            'Settings dialogue',
            'Icon',
            'Setting to choose the type of the icon on the tray'
        )
        tray_temp = QCoreApplication.translate(
            'Settings dialogue',
            'Temperature',
            'Setting to choose the type of the icon on the tray'
        )
        tray_icon_feels_like = QCoreApplication.translate(
            'Settings dialogue',
            'Icon & Feels like temperature',
            'Setting to choose the type of the icon on the tray'
        )
        tray_feels_like = QCoreApplication.translate(
            'Settings dialogue',
            'Feels like temperature',
            'Setting to choose the type of the icon on the tray'
        )
        self.tray_dico = {
            'icon&temp': tray_icon_temp,
            'icon': tray_icon,
            'temp': tray_temp,
            'icon&feels_like': tray_icon_feels_like,
            'feels_like_temp': tray_feels_like,
        }
        set_tray_icon = self.settings.value('TrayType') or 'icon&temp'
        tray_icon_list = sorted(self.tray_dico.values())
        self.tray_icon_combo.addItems(tray_icon_list)
        self.tray_icon_combo.setCurrentIndex(
            self.tray_icon_combo.findText(self.tray_dico[set_tray_icon])
        )
        self.tray_icon_combo.currentIndexChanged.connect(self.tray)
        self.tray_changed = False
        # Weather icons
        self.comboBox_icons_theme = QComboBox()
        thema_list = ['OpenWeatherMap']
        self.system_default_theme_translated = {
            'System default': QCoreApplication.translate(
                'Settings dialogue',
                'System default',
                'ComboBox to choose the system default icons theme'
            )
        }
        self.comboBox_icons_theme.setToolTip(
            QCoreApplication.translate(
                'Settings dialogue',
                'Icons theme',
                'Tooltip of the ComboBox to choose the icons theme'
            )
        )
        thema_list.append(self.system_default_theme_translated['System default'])
        logging.debug('Icons theme paths : ' + str(QIcon.themeSearchPaths()))
        for themedir in QIcon.themeSearchPaths():
            for dirpath in glob.glob(themedir + '/*/'):
                for path in glob.glob(dirpath + '/*'):
                    if path.count('index.theme'):
                        thema = os.path.basename(os.path.dirname(path))
                        if thema not in thema_list:
                            thema_list.append(thema)

        thema_list = sorted(thema_list, key=str.casefold)
        logging.debug('Found themes : ' + str(thema_list))
        self.comboBox_icons_theme.addItems(thema_list)

        icontheme_conf = self.settings.value('IconsTheme') or 'System default'
        if icontheme_conf == 'System default':
            self.comboBox_icons_theme.setCurrentIndex(
                self.comboBox_icons_theme.findText(self.system_default_theme_translated['System default'])
            )
        else:
            self.comboBox_icons_theme.setCurrentIndex(
                self.comboBox_icons_theme.findText(icontheme_conf)
            )

        self.comboBox_icons_theme.currentIndexChanged.connect(self.system_theme_icons)
        self.comboBox_icons_theme_changed = False

        # Tray icon initialization size and temp position
        self.tray_icon_init_label = QLabel(
            QCoreApplication.translate(
                'Settings dialogue',
                'Tray icon size and text position',
                'This option concernes 2 dropdown lists'
                'to define the size to initialize the tray icon'
                'and the position of the temperature text in the icon'
            )
        )
        self.tray_icon_init_label.setToolTip(
            QCoreApplication.translate(
                'Settings dialogue',
                'Changing this setting you can improve the displaying of the temperature in the tray icon',
                'ToolTip of the label "Tray icon size and text position"'
            )
        )

        self.tray_icon_init_cmb = QComboBox()
        self.tray_icon_init_cmb.setToolTip(
            QCoreApplication.translate(
                'Settings dialogue',
                'Tray icon initialization size (default size: 64x64)',
                'Tooltip of the drowndrop list with icon sizes'
            )
        )
        self.tray_icon_init_cmb.addItems(["16x16", "24x24", "32x32", "64x64"])
        icon_init_size = self.settings.value('Tray_icon_init_size') or '64x64'
        self.tray_icon_init_cmb.setCurrentText(icon_init_size)
        self.tray_icon_init_cmb.currentIndexChanged.connect(self.tray_icon_init_size_change)
        self.tray_icon_init_cmb_changed = False

        self.tray_icon_temp_pos_spinbox = QSpinBox()
        self.tray_icon_temp_pos_spinbox.setRange(-20, 20)
        temp_pos_in_icon = self.settings.value('Tray_icon_temp_position') or '-12'
        self.tray_icon_temp_pos_spinbox.setValue(int(temp_pos_in_icon))
        self.tray_icon_temp_pos_spinbox_changed = False
        self.tray_icon_temp_pos_spinbox.valueChanged.connect(self.tray_icon_temp_pos_change)
        self.tray_icon_temp_pos_spinbox.setToolTip(
            QCoreApplication.translate(
                'Settings dialogue',
                'Temperature position in the icon (vertically) (default value: -12)',
                'ToolTip of the widget (a spinbox) to define the position of the temperature paint in the tray icon'
            )
        )

        # Toggle icon Temp
        self.toggle_tray_label = QLabel(
            QCoreApplication.translate(
                'Settings dialogue',
                'Toggle tray icon interval',
                'Label for the option of the checkbox to '
                'activate the toggle of the tray icon and temperature'
            )
        )
        toggle_interval = self.settings.value('Toggle_tray_interval') or '0'
        self.toggle_tray_spinbox = QSpinBox()
        self.toggle_tray_spinbox.setRange(0, 100000)
        self.toggle_tray_spinbox_changed = False
        self.toggle_tray_spinbox.valueChanged.connect(self.toggle_tray_interval_change)
        self.toggle_tray_spinbox.setValue(int(toggle_interval))
        self.toggle_tray_interval_label = QLabel(
            QCoreApplication.translate(
                'Settings dialogue',
                'seconds. Set to 0 to deactivate',
                'Label after the spinbox to choose the interval '
                'to toggle the tray icon and temperature'
            )
        )
        self.activate_toggle_check()
        # Font of temperature in tray
        self.font_tray_changed = False
        self.font_tray_conf_new = ''
        self.font_tray_conf = self.settings.value('FontTray') or False
        if not self.font_tray_conf:
            self.font_tray_conf = 'Sans Serif,18,-1,5,50,0,0,0,0,0'
            self.settings.setValue('FontTray', self.font_tray_conf)
        self.font_tray_label = QLabel(
            QCoreApplication.translate(
                'Settings dialogue',
                'Temperature font in system tray',
                'Setting for the font size of the temperature in the tray icon'
            )
        )
        font_tray_btn_label = (
            f"{self.font_tray_conf.split(',')[0]} - "
            f"{self.font_tray_conf.split(',')[1]} - "
            f"{self.font_tray_conf.split(',')[-1]}"
        )
        self.font_tray_btn = QPushButton(font_tray_btn_label)
        self.font_tray_btn.clicked.connect(self.getfont)
        # Proxy
        self.proxy_label = QLabel(
            QCoreApplication.translate(
                'Checkbox',
                'Connection by proxy',
                'Settings dialogue'
            )
        )
        self.proxy_chbox = QCheckBox()
        proxy_bool = self.settings.value('Proxy') or 'False'
        self.proxy_bool = eval(proxy_bool)
        self.proxy_chbox.setChecked(self.proxy_bool)
        self.proxy_chbox.stateChanged.connect(self.proxy)
        self.proxy_changed = False
        self.proxy_button = QPushButton(
            QCoreApplication.translate(
                'Label of button to open the proxy dialogue',
                'Settings',
                'Settings dialogue'
            )
        )
        self.proxy_button.clicked.connect(self.proxy_settings)
        self.proxy_button.setEnabled(self.proxy_bool)
        # Openweathermap key
        self.owmkey_label = QLabel(
            QCoreApplication.translate(
                'The key that user can generate in his OpenWeatherMap profile',
                'OpenWeatherMap key',
                'Settings dialogue'
            )
        )
        self.owmkey_create = QLabel(
            QCoreApplication.translate(
                'Link to create a profile in OpenWeatherMap',
                "<a href=\"http://home.openweathermap.org/users/sign_up\">Create key</a>",
                'Settings dialogue'
            )
        )
        self.owmkey_create.setOpenExternalLinks(True)
        apikey = self.settings.value('APPID') or '3598b2b9cf09e6aed63225daf32a7293'
        self.owmkey_text = QLineEdit()
        self.owmkey_text.setToolTip(
            QCoreApplication.translate(
                'Settings dialogue',
                'It is suggested to use a personal key.<br/>If someone hijacks the default meteo-qt key you will not be able to get the weather data (error 401 unauthorized)',
                'Tooltip of the api key text field'
            )
        )
        self.owmkey_text.setText(apikey)
        self.owmkey_text.textChanged.connect(self.apikey_changed)

        self.start_minimized_label = QLabel(
            QCoreApplication.translate(
                'Checkable option to show or not the window at startup',
                'Start minimized',
                'Settings dialogue'
            )
        )
        self.start_minimized_chbx = QCheckBox()
        start_minimized_bool = self.settings.value('StartMinimized') or 'True'
        self.start_minimized_bool = eval(start_minimized_bool)
        self.start_minimized_chbx.setChecked(self.start_minimized_bool)
        self.start_minimized_chbx.stateChanged.connect(self.start_minimized)
        self.start_minimized_changed = False

        self.logging_label = QLabel(
            QCoreApplication.translate(
                'Option for logging level',
                'Logging level',
                'Settings window'
            )
        )
        self.logging_level_combo = QComboBox()
        logging_levels = ['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG']
        self.logging_level_combo.addItems(logging_levels)
        level = self.settings.value('Logging/Level') or 'INFO'
        self.logging_level_combo.setCurrentIndex(
            self.logging_level_combo.findText(level)
        )
        self.logging_level_combo.currentIndexChanged.connect(self.logging_set)
        self.logging_changed = False

        # ----------
        self.panel = QGridLayout()
        self.panel.addWidget(self.city_title, 0, 0)
        self.panel.addWidget(self.city_combo, 0, 1)
        self.panel.addWidget(self.city_button, 0, 2)
        self.panel.addWidget(self.language_label, 1, 0)
        self.panel.addWidget(self.language_combo, 1, 1)
        self.panel.addWidget(self.units_label, 2, 0)
        self.panel.addWidget(self.units_combo, 2, 1)
        self.panel.addWidget(self.wind_unit_label, 3, 0)
        self.panel.addWidget(self.wind_unit_combo, 3, 1)
        self.panel.addWidget(self.temp_decimal_label, 4, 0)
        self.panel.addWidget(self.temp_decimal_combo, 4, 1)
        self.panel.addWidget(self.interval_label, 5, 0)
        self.panel.addWidget(self.interval_combo, 5, 1)
        self.panel.addWidget(self.interval_min, 5, 2)
        self.panel.addWidget(self.autostart_label, 6, 0)
        self.panel.addWidget(self.autostart_checkbox, 6, 1)
        self.panel.addWidget(self.temp_colorLabel, 7, 0)
        self.panel.addWidget(self.temp_colorButton, 7, 1)
        self.panel.addWidget(self.temp_color_resetButton, 7, 2)
        self.panel.addWidget(self.notifier_label, 8, 0)
        self.panel.addWidget(self.notifier_checkbox, 8, 1)
        self.panel.addWidget(self.tray_icon_temp_label, 9, 0)
        self.panel.addWidget(self.tray_icon_combo, 9, 1)
        self.panel.addWidget(self.comboBox_icons_theme, 9, 2)
        self.panel.addWidget(self.tray_icon_init_label, 10, 0)
        self.panel.addWidget(self.tray_icon_init_cmb, 10, 1)
        self.panel.addWidget(self.tray_icon_temp_pos_spinbox, 10, 2)
        self.panel.addWidget(self.toggle_tray_label, 11, 0)
        self.panel.addWidget(self.toggle_tray_spinbox, 11, 1)
        self.panel.addWidget(self.toggle_tray_interval_label, 11, 2)
        self.panel.addWidget(self.font_tray_label, 12, 0)
        self.panel.addWidget(self.font_tray_btn, 12, 1)
        self.panel.addWidget(self.proxy_label, 13, 0)
        self.panel.addWidget(self.proxy_chbox, 13, 1)
        self.panel.addWidget(self.proxy_button, 13, 2)
        self.panel.addWidget(self.owmkey_label, 14, 0)
        self.panel.addWidget(self.owmkey_text, 14, 1)
        self.panel.addWidget(self.owmkey_create, 14, 2)
        self.panel.addWidget(self.start_minimized_label, 15, 0)
        self.panel.addWidget(self.start_minimized_chbx, 15, 1)
        self.panel.addWidget(self.logging_label, 16, 0)
        self.panel.addWidget(self.logging_level_combo, 16, 1)

        self.layout.addLayout(self.panel)
        self.layout.addLayout(self.buttonLayout)
        self.statusbar = QLabel()
        self.layout.addWidget(self.statusbar)
        self.nokey_message = QCoreApplication.translate(
            'Warning message after pressing Ok',
            'Please enter your OpenWeatherMap key',
            'Settings dialogue'
        )
        self.nocity_message = QCoreApplication.translate(
            'Warning message after pressing OK',
            'Please add a city',
            'Settings dialogue'
        )
        self.setLayout(self.layout)
        self.setWindowTitle(self.tr('Meteo-qt Configuration'))

    def wind_unit_change_apply(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.wind_unit_changed = True

    def units(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.units_changed = True
        unit = self.units_combo.currentText()
        setUnit = [
            key for key, value in self.units_dico.items() if value == unit
        ]
        if setUnit[0] == 'metric':
            self.wind_unit_combo.model().item(2).setEnabled(True)
        else:
            self.wind_unit_combo.model().item(2).setEnabled(False)
            wind = self.wind_unit_combo.currentText()
            setWind = [
                key for key, value in self.wind_unit_dico.items() if value == wind
            ]
            if setWind[0] == 'km':
                self.wind_unit_combo.setCurrentIndex(
                    self.wind_unit_combo.findText(self.wind_unit_dico['df'])
                )

    def language(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.lang_changed = True

    def city_default(self):
        allitems = [
            self.city_combo.itemText(i) for i in range(self.city_combo.count())
        ]
        allitems_not_translated = []
        for i in allitems:
            allitems_not_translated.append(self.find_city_key(i))
        city_name = self.city_combo.currentText()
        city_name = self.find_city_key(city_name)
        citytosave = city_name.split('_')
        # This self variable will serve to check if a translation
        # exist for the current city when quitting
        self.citytosave = '_'.join(citytosave)
        if len(citytosave) < 3:
            return
        self.id_before = citytosave[2]
        self.city_before = citytosave[0]
        self.country_before = citytosave[1]
        self.city_list_before = allitems_not_translated[:]
        self.city_list_before.pop(self.city_list_before.index(city_name))
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def interval(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.interval_changed = True

    def edit_cities_list(self):
        apikey = self.owmkey_text.text()
        apiid = '&APPID=' + apikey
        if apikey == '':
            self.statusbar.setText(self.nokey_message)
            return
        dialog = citylistdlg.CityListDlg(
            self.citylist, self.accurate_url, apiid,
            self.trans_cities_dict, self
        )
        dialog.citieslist_signal.connect(self.cities_list)
        dialog.citiesdict_signal.connect(self.cities_dict)
        dialog.exec()

    def cities_dict(self, cit_dict):
        self.trans_cities_dict = cit_dict

    def cities_list(self, cit_list):
        if len(cit_list) > 0:
            citytosave = cit_list[0].split('_')
            self.id_before = citytosave[2]
            self.city_before = citytosave[0]
            self.country_before = citytosave[1]
            if len(cit_list) > 1:
                self.city_list_before = cit_list[1:]
            else:
                self.city_list_before = str('')
        else:
            self.id_before = ''
            self.city_before = ''
            self.country_before = ''
            self.city_list_before = []
            self.clear_combo = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.first = False
        self.add_cities_incombo()

    def autostart(self, state):
        self.autostart_state = state
        self.autostart_changed = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def autostart_apply(self):
        dir_auto = '/.config/autostart/'
        d_file = 'meteo-qt.desktop'
        home = os.getenv('HOME')
        total_path = home + dir_auto + d_file
        if self.autostart_state == 2:
            desktop_file = [
                '[Desktop Entry]\n',
                'Exec=meteo-qt\n',
                'Name=meteo-qt\n',
                'Type=Application\n',
                'Version=1.0\n',
                'X-LXQt-Need-Tray=true\n'
            ]
            if not os.path.exists(home + dir_auto):
                os.system('mkdir -p {}'.format(os.path.dirname(total_path)))
            with open(total_path, 'w') as out_file:
                out_file.writelines(desktop_file)
            self.settings.setValue('Autostart', 'True')
            logging.debug('Write desktop file in ~/.config/autostart')
        elif self.autostart_state == 0:
            if os.path.exists(total_path):
                os.remove(total_path)
            self.settings.setValue('Autostart', 'False')
            logging.debug('Remove desktop file from ~/.config/autostart')
        else:
            return

    def color_chooser(self):
        current_color = self.temp_tray_color
        if hasattr(self, 'color_before'):
            current_color = self.color_before
        col = QColorDialog.getColor(QColor(current_color), self)
        if col.isValid():
            self.temp_colorButton.setStyleSheet(
                'QWidget {{ background-color: {0} }}'.format(col.name())
            )
            # focus to next elem to show immediatley the colour
            # in the button (in some DEs)
            self.temp_color_resetButton.setFocus()
            self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
            self.color_before = col.name()
        else:
            logging.debug('Invalid color:' + str(col))
            self.temp_color_resetButton.setFocus()

    def color_reset(self):
        self.temp_colorButton.setStyleSheet('QWidget { background-color:  }')
        self.color_before = ''
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def notifier(self, state):
        self.notifier_state = state
        self.notifier_changed = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def notifier_apply(self):
        if self.notifier_state == 2:
            self.settings.setValue('Notifications', 'True')
            logging.debug('Write: Notifications = True')
        elif self.notifier_state == 0:
            self.settings.setValue('Notifications', 'False')
            logging.debug('Write: Notifications = False')

    def temp_decimal(self, state):
        self.temp_decimal_state = state
        self.temp_decimal_changed = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def activate_toggle_check(self):
        current_tray_option = self.tray_icon_combo.currentText()
        for key, value in self.tray_dico.items():
            if value == current_tray_option:
                if key == 'icon&temp' or key == 'icon&feels_like':
                    self.toggle_tray_spinbox.setEnabled(True)
                else:
                    self.toggle_tray_spinbox.setEnabled(False)

    def tray(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.tray_changed = True
        self.activate_toggle_check()

    def tray_apply(self):
        tray = self.tray_icon_combo.currentText()
        self.settings.setValue('Tray', tray)
        logging.debug('Write >' + 'Tray >' + str(tray))
        settray = [
            key for key, value in self.tray_dico.items() if value == tray
        ]
        self.settings.setValue('TrayType', settray[0])

    def tray_icon_init_size_change(self):
        self.tray_icon_init_cmb_changed = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def tray_icon_init_size_apply(self):
        self.settings.setValue('Tray_icon_init_size', self.tray_icon_init_cmb.currentText())

    def tray_icon_temp_pos_change(self, pos):
        self.tray_icon_temp_pos_spinbox_changed = True
        self.tray_icon_temp_pos = pos
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def tray_icon_temp_pos_apply(self):
        self.settings.setValue('Tray_icon_temp_position', str(self.tray_icon_temp_pos))

    def toggle_tray_interval_change(self, interval):
        self.toggle_tray_spinbox_changed = True
        self.toggle_tray_interval = interval
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def toggle_tray_interval_apply(self):
        self.settings.setValue('Toggle_tray_interval', str(self.toggle_tray_interval))

    def getfont(self):
        if self.font_tray_conf_new != "":
            current_font = self.font_tray_conf_new
        else:
            current_font = self.font_tray_conf
        ff = QFont()
        ff.fromString(current_font)
        font, ok = QFontDialog.getFont(ff, self)
        if ok and font.toString() != self.font_tray_conf:
            self.font_tray_changed = True
            self.font_tray_conf_new = font.toString()
            self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
            font_tray_btn_label = (
                f"{self.font_tray_conf_new.split(',')[0]} - "
                f"{self.font_tray_conf_new.split(',')[1]} - "
                f"{self.font_tray_conf_new.split(',')[-1]}"
            )
            self.font_tray_btn.setText(font_tray_btn_label)

    def font_tray_apply(self):
        logging.debug(f'Apply font for tray: {self.font_tray_conf_new}')
        self.settings.setValue('FontTray', self.font_tray_conf_new)

    def system_theme_icons(self):
        self.comboBox_icons_theme_changed = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def system_icontheme_apply(self):
        if self.comboBox_icons_theme.currentText() == self.system_default_theme_translated['System default']:
            self.settings.setValue('IconsTheme', 'System default')
        else:
            self.settings.setValue('IconsTheme', self.comboBox_icons_theme.currentText())

    def start_minimized(self, state):
        self.start_minimized_state = state
        self.start_minimized_changed = True
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def start_minimized_apply(self):
        if self.start_minimized_state == 2:
            start_minimized = 'True'
        else:
            start_minimized = 'False'
        self.settings.setValue('StartMinimized', start_minimized)

    def logging_set(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        self.logging_changed = True

    def logging_level_apply(self):
        level = self.logging_level_combo.currentText()
        self.settings.setValue('Logging/Level', level)

    def proxy(self, state):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)
        if state == 2:
            self.proxy_bool = True
            self.proxy_button.setEnabled(True)
        else:
            self.proxy_bool = False
            self.proxy_button.setEnabled(False)

    def proxy_settings(self):
        dialog = proxydlg.Proxy(self)
        dialog.exec()

    def apikey_changed(self):
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(True)

    def apply_settings(self):
        self.accepted()

    def clear_translations(self):
        ''' Save the list of the current cities list
            and remove the odd or blank translations'''
        self.settings.setValue('CityList', str(self.citylist))
        translations_to_delete = []
        for key, value in self.trans_cities_dict.items():
            if key == value or value == '' or key not in self.citylist:
                translations_to_delete.append(key)
        for i in translations_to_delete:
            del self.trans_cities_dict[i]
        self.settings.setValue(
            'CitiesTranslation', str(self.trans_cities_dict)
        )
        logging.debug(
            'write ' + 'CitiesTranslation ' + str(self.trans_cities_dict)
        )

    def accepted(self):
        self.clear_translations()
        apikey = self.owmkey_text.text()
        city_name = self.city_combo.currentText()
        if apikey == '':
            self.statusbar.setText(self.nokey_message)
            return
        else:
            self.statusbar.setText('')
            self.settings.setValue('APPID', str(self.owmkey_text.text()))
        if city_name == '':
            self.statusbar.setText(self.nocity_message)
            return
        else:
            self.statusbar.setText('')
        self.buttonBox.button(QDialogButtonBox.StandardButton.Apply).setEnabled(False)
        if hasattr(self, 'id_before'):
            self.settings.setValue('ID', self.id_before)
            logging.debug('write ' + 'ID' + str(self.id_before))
        if hasattr(self, 'city_before'):
            self.settings.setValue('City', self.city_before)
            logging.debug('write ' + 'City' + str(self.city_before))
        if hasattr(self, 'country_before'):
            self.settings.setValue('Country', self.country_before)
            logging.debug('write ' + 'Country' + str(self.country_before))
        if hasattr(self, 'color_before'):
            self.settings.setValue('TrayColor', self.color_before)
            if self.color_before == '':
                self.color_before = 'None'
            logging.debug(
                'Write font color for temp in tray: {0}'.format(self.color_before)
            )
        if self.autostart_changed:
            self.autostart_apply()
        if self.interval_changed:
            time = self.interval_combo.currentText()
            self.settings.setValue('Interval', time)
            logging.debug('Write ' + 'Interval ' + str(time))
        if self.lang_changed:
            lang = self.language_combo.currentText()
            setlang = [
                key for key, value in self.language_dico.items() if value == lang
            ]
            self.settings.setValue('Language', setlang[0])
            logging.debug('Write ' + 'Language ' + str(setlang[0]))
        if self.units_changed:
            unit = self.units_combo.currentText()
            setUnit = [
                key for key, value in self.units_dico.items() if value == unit
            ]
            self.settings.setValue('Unit', setUnit[0])
            logging.debug('Write ' + 'Unit ' + str(setUnit[0]))
        if self.wind_unit_changed:
            wind = self.wind_unit_combo.currentText()
            setWind = [
                key for key, value in self.wind_unit_dico.items() if value == wind
            ]
            logging.debug('Write ' + 'Wind_unit ' + str(setWind[0]))
            self.settings.setValue('Wind_unit', str(setWind[0]))
        if self.temp_decimal_changed:
            decimal = self.temp_decimal_combo.currentText()
            decimal_bool_str = 'False'
            if decimal == '0.1°':
                decimal_bool_str = 'True'
            self.settings.setValue('Decimal', decimal_bool_str)
            logging.debug('Write: Decimal in tray icon = ' + decimal_bool_str)
        if self.notifier_changed:
            self.notifier_apply()
        if self.tray_changed:
            self.tray_apply()
        if self.tray_icon_init_cmb_changed:
            self.tray_icon_init_size_apply()
        if self.tray_icon_temp_pos_spinbox_changed:
            self.tray_icon_temp_pos_apply()
        if self.toggle_tray_spinbox_changed:
            self.toggle_tray_interval_apply()
        if self.font_tray_changed:
            self.font_tray_apply()
        if self.comboBox_icons_theme_changed:
            self.system_icontheme_apply()
        if self.start_minimized_changed:
            self.start_minimized_apply()
        if self.logging_changed:
            self.logging_level_apply()
        proxy_url = self.settings.value('Proxy_url') or ''
        if proxy_url == '':
            self.proxy_bool = False
        self.settings.setValue('Proxy', str(self.proxy_bool))
        self.applied_signal.emit()

    def accept(self):
        self.accepted()
        apikey = self.owmkey_text.text()
        city_name = self.city_combo.currentText()
        if apikey == '':
            self.statusbar.setText(self.nokey_message)
            return
        if city_name == '':
            self.statusbar.setText(self.nocity_message)
            return
        QDialog.accept(self)

    def add_cities_incombo(self):
        list_cities = ''
        self.city_combo.clear()
        if self.clear_combo:
            return
        if self.first:
            list_cities = self.settings.value('CityList')
            if list_cities is not None:
                self.city_list_before = list_cities[:]
            self.citylist = [
                self.set_city + '_'
                + self.settings.value('Country') + '_'
                + self.settings.value('ID')
            ]
        else:
            self.citylist = [
                self.city_before + '_' + self.country_before
                + '_' + self.id_before
            ]
            list_cities = self.city_list_before[:]
        if list_cities is None:
            list_cities = []
        if list_cities != '' and list_cities is not None:
            if type(list_cities) is str:
                list_cities = eval(list_cities)
            self.citylist = self.citylist + list_cities
        duplicate = []
        for i in self.citylist:
            if i not in duplicate:
                duplicate.append(i)
        self.citylist = duplicate[:]
        self.translated = []
        for city in self.citylist:
            self.translated.append(self.trans_cities_dict.get(city, city))
        self.city_combo.addItems(self.translated)
        if len(list_cities) > 0:
            maxi = len(max(list_cities, key=len))
            self.city_combo.setMinimumSize(maxi * 8, 23)

    def find_city_key(self, city):
        for key, value in self.trans_cities_dict.items():
            if value == city:
                return key
        return city



from PyQt6.QtCore import QCoreApplication, QSettings, Qt, pyqtSignal
from PyQt6.QtWidgets import (QDialog, QDialogButtonBox, QGridLayout,
                             QHBoxLayout, QLabel, QLineEdit, QVBoxLayout)


class CityTranslate(QDialog):
    city_signal = pyqtSignal([dict])

    def __init__(self, city, cities_list, parent=None):
        super(CityTranslate, self).__init__(parent)
        self.city = city
        self.settings = QSettings()
        self.trans_cities_dict = cities_list
        self.layout = QVBoxLayout()
        self.buttonLayout = QHBoxLayout()
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
        )
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonLayout.addWidget(self.buttonBox)
        self.untranslate_city_label = QLabel(self.find_city_key(self.city))
        self.translate_line = QLineEdit(self.city)
        self.translate_line.selectAll()
        self.translate_line.setMinimumWidth(300)
        self.status_layout = QHBoxLayout()
        self.status_label = QLabel()
        self.status_layout.addWidget(self.status_label)
        self.panel = QGridLayout()
        self.panel.addWidget(self.untranslate_city_label, 0, 0)
        self.panel.addWidget(self.translate_line, 1, 0)
        self.layout.addLayout(self.panel)
        self.layout.addLayout(self.status_layout)
        self.layout.addLayout(self.buttonLayout)
        self.setLayout(self.layout)
        self.setWindowTitle(QCoreApplication.translate('Window title',
                            'City translation', 'City translation dialogue'))

    def find_city_key(self, city):
        for key, value in self.trans_cities_dict.items():
            if value == city:
                return key
        return city

    def accept(self):
        city_dict = {}
        current_city = self.translate_line.text()
        for city, trans in self.trans_cities_dict.items():
            if (
                current_city == trans
                and city != self.untranslate_city_label.text()
            ):
                self.status_label.setText(
                    QCoreApplication.translate(
                        'Warning message in dialog status bar',
                        'The city already exist',
                        'City translation'
                    )
                )
                return

        city_dict[self.untranslate_city_label.text()] = (
            self.translate_line.text()
        )

        self.city_signal[dict].emit(city_dict)
        QDialog.accept(self)
